searchNodes=[{"doc":"PostgreSQL driver for Elixir. Postgrex is a partial implementation of the Postgres frontend/backend message protocol . It performs wire messaging in Elixir, as opposed to binding to a library such as libpq in C. A Postgrex query is performed as &quot; extended query &quot;. An &quot;extended query&quot; involves separate server-side parse, bind, and execute stages, each of which may be re-used for efficiency. For example, libraries like Ecto caches queries, so a query only has to be parsed and planned once. This is all done via wire messaging, without relying on PREPARE q AS (...) and EXECUTE q() SQL statements directly. This module handles the connection to PostgreSQL, providing support for queries, transactions, connection backoff, logging, pooling and more. Note that the notifications API (pub/sub) supported by PostgreSQL is handled by Postgrex.Notifications . Hence, to use this feature, you need to start a separate (notifications) connection.","ref":"Postgrex.html","title":"Postgrex","type":"module"},{"doc":"Returns a supervisor child specification for a DBConnection pool.","ref":"Postgrex.html#child_spec/1","title":"Postgrex.child_spec/1","type":"function"},{"doc":"Closes an (extended) prepared query and returns :ok or {:error, %Postgrex.Error{}} if there was an error. Closing a query releases any resources held by postgresql for a prepared query with that name. See Postgrex.Query for the query data. This function may still raise an exception if there is an issue with types ( ArgumentError ), connection ( DBConnection.ConnectionError ), ownership ( DBConnection.OwnershipError ) or other error ( RuntimeError ). Options :queue - Whether to wait for connection in a queue (default: true ); :timeout - Close request timeout (default: 15000 ); :mode - set to :savepoint to use a savepoint to rollback to before the close on error, otherwise set to :transaction (default: :transaction ); Examples query = Postgrex . prepare! ( conn , &quot;&quot; , &quot;CREATE TABLE posts (id serial, title text)&quot; ) Postgrex . close ( conn , query )","ref":"Postgrex.html#close/3","title":"Postgrex.close/3","type":"function"},{"doc":"Closes an (extended) prepared query and returns :ok or raises Postgrex.Error if there was an error. See close/3 .","ref":"Postgrex.html#close!/3","title":"Postgrex.close!/3","type":"function"},{"doc":"Runs an (extended) prepared query. It returns the result as {:ok, %Postgrex.Query{}, %Postgrex.Result{}} or {:error, %Postgrex.Error{}} if there was an error. Parameters are given as part of the prepared query, %Postgrex.Query{} . See the README for information on how Postgrex encodes and decodes Elixir values by default. See Postgrex.Query for the query data and Postgrex.Result for the result data. Options :queue - Whether to wait for connection in a queue (default: true ); :timeout - Execute request timeout (default: 15000 ); :decode_mapper - Fun to map each row in the result to a term after decoding, (default: fn x -&gt; x end ); :mode - set to :savepoint to use a savepoint to rollback to before the execute on error, otherwise set to :transaction (default: :transaction ); Examples query = Postgrex . prepare! ( conn , &quot;&quot; , &quot;CREATE TABLE posts (id serial, title text)&quot; ) Postgrex . execute ( conn , query , [ ] ) query = Postgrex . prepare! ( conn , &quot;&quot; , &quot;SELECT id FROM posts WHERE title like $1&quot; ) Postgrex . execute ( conn , query , [ &quot;%my%&quot; ] )","ref":"Postgrex.html#execute/4","title":"Postgrex.execute/4","type":"function"},{"doc":"Runs an (extended) prepared query and returns the result or raises Postgrex.Error if there was an error. See execute/4 .","ref":"Postgrex.html#execute!/4","title":"Postgrex.execute!/4","type":"function"},{"doc":"Returns a cached map of connection parameters. Options :timeout - Call timeout (default: 15000 )","ref":"Postgrex.html#parameters/2","title":"Postgrex.parameters/2","type":"function"},{"doc":"Prepares an (extended) query. It returns the result as {:ok, %Postgrex.Query{}} or {:error, %Postgrex.Error{}} if there was an error. Parameters can be set in the query as $1 embedded in the query string. To execute the query call execute/4 . To close the prepared query call close/3 . See Postgrex.Query for the query data. This function may still raise an exception if there is an issue with types ( ArgumentError ), connection ( DBConnection.ConnectionError ), ownership ( DBConnection.OwnershipError ) or other error ( RuntimeError ). For unnamed prepared statements, pass an empty string for name . This can be useful when trying to avoid the generic query plan Postgres creates for named prepared statements. You may also set prepare: :unnamed at the connection level so that every prepared statement using that connection will be unnamed. Options :queue - Whether to wait for connection in a queue (default: true ); :timeout - Prepare request timeout (default: 15000 ); :mode - set to :savepoint to use a savepoint to rollback to before the prepare on error, otherwise set to :transaction (default: :transaction ); Examples Postgrex . prepare ( conn , &quot;&quot; , &quot;CREATE TABLE posts (id serial, title text)&quot; )","ref":"Postgrex.html#prepare/4","title":"Postgrex.prepare/4","type":"function"},{"doc":"Prepares an (extended) query and returns the prepared query or raises Postgrex.Error if there was an error. See prepare/4 .","ref":"Postgrex.html#prepare!/4","title":"Postgrex.prepare!/4","type":"function"},{"doc":"Prepares and executes a query in a single step. It returns the result as {:ok, %Postgrex.Query{}, %Postgrex.Result{}} or {:error, %Postgrex.Error{}} if there was an error. Parameters are given as part of the prepared query, %Postgrex.Query{} . For unnamed prepared statements, pass an empty string for name . This can be useful when trying to avoid the generic query plan Postgres creates for named prepared statements. You may also set prepare: :unnamed at the connection level so that every prepared statement using that connection will be unnamed. See the README for information on how Postgrex encodes and decodes Elixir values by default. See Postgrex.Query for the query data and Postgrex.Result for the result data. Options :queue - Whether to wait for connection in a queue (default: true ); :timeout - Execute request timeout (default: 15000 ); :decode_mapper - Fun to map each row in the result to a term after decoding, (default: fn x -&gt; x end ); :mode - set to :savepoint to use a savepoint to rollback to before the execute on error, otherwise set to :transaction (default: :transaction ); Examples Postgrex . prepare_execute ( conn , &quot;&quot; , &quot;SELECT id FROM posts WHERE title like $1&quot; , [ &quot;%my%&quot; ] )","ref":"Postgrex.html#prepare_execute/5","title":"Postgrex.prepare_execute/5","type":"function"},{"doc":"Prepares and runs a query and returns the result or raises Postgrex.Error if there was an error. See prepare_execute/5 .","ref":"Postgrex.html#prepare_execute!/5","title":"Postgrex.prepare_execute!/5","type":"function"},{"doc":"Runs an (extended) query and returns the result as {:ok, %Postgrex.Result{}} or {:error, %Postgrex.Error{}} if there was a database error. Parameters can be set in the query as $1 embedded in the query string. Parameters are given as a list of elixir values. See the README for information on how Postgrex encodes and decodes Elixir values by default. See Postgrex.Result for the result data. This function may still raise an exception if there is an issue with types ( ArgumentError ), connection ( DBConnection.ConnectionError ), ownership ( DBConnection.OwnershipError ) or other error ( RuntimeError ). Options :queue - Whether to wait for connection in a queue (default: true ); :timeout - Query request timeout (default: 15000 ); :decode_mapper - Fun to map each row in the result to a term after decoding, (default: fn x -&gt; x end ); :mode - set to :savepoint to use a savepoint to rollback to before the query on error, otherwise set to :transaction (default: :transaction ); :cache_statement - Caches the query with the given name Examples Postgrex . query ( conn , &quot;CREATE TABLE posts (id serial, title text)&quot; , [ ] ) Postgrex . query ( conn , &quot;INSERT INTO posts (title) VALUES (&#39;my title&#39;)&quot; , [ ] ) Postgrex . query ( conn , &quot;SELECT title FROM posts&quot; , [ ] ) Postgrex . query ( conn , &quot;SELECT id FROM posts WHERE title like $1&quot; , [ &quot;%my%&quot; ] ) Postgrex . query ( conn , &quot;COPY posts TO STDOUT&quot; , [ ] )","ref":"Postgrex.html#query/4","title":"Postgrex.query/4","type":"function"},{"doc":"Runs an (extended) query and returns the result or raises Postgrex.Error if there was an error. See query/3 .","ref":"Postgrex.html#query!/4","title":"Postgrex.query!/4","type":"function"},{"doc":"Rollback a transaction, does not return. Aborts the current transaction fun. If inside multiple transaction/3 functions, bubbles up to the top level. Example { :error , :oops } = Postgrex . transaction ( pid , fn ( conn ) -&gt; DBConnection . rollback ( conn , :bar ) IO . puts &quot;never reaches here!&quot; end )","ref":"Postgrex.html#rollback/2","title":"Postgrex.rollback/2","type":"function"},{"doc":"Start the connection process and connect to postgres. Options Postgrex provides multiple ways to connect to the server, listed in order of precedence below: :hostname - Server hostname (default: PGHOST env variable, then localhost); :port - Server port (default: PGPORT env variable, then 5432); :endpoints - A list of endpoints (host and port pairs, with an optional extra_opts keyword list); Postgrex will try each endpoint in order, one by one, until the connection succeeds; The syntax is [{host1, port1},{host2, port2},{host3, port3}] or [{host1, port1, extra_opt1: value},{host2, port2, extra_opt2: value}}] ; This option takes precedence over :hostname+:port ; :socket_dir - Connect to PostgreSQL via UNIX sockets in the given directory; The socket name is derived based on the port. This is the preferred method for configuring sockets and it takes precedence over the hostname. If you are connecting to a socket outside of the PostgreSQL convention, use :socket instead; :socket - Connect to PostgreSQL via UNIX sockets in the given path. This option takes precedence over the :hostname , :endpoints and :socket_dir ; Once a server is specified, you can configure the connection with the following: :database - Database (default: PGDATABASE env variable; otherwise required); :username - Username (default: PGUSER env variable, then USER env var); :password - User password (default: PGPASSWORD env variable); :parameters - Keyword list of connection parameters; :timeout - Socket receive timeout when idle in milliseconds (default: 15000 ); :connect_timeout - Socket connect timeout in milliseconds (defaults to :timeout value); :handshake_timeout - Connection handshake timeout in milliseconds (defaults to :timeout value); :ping_timeout - Socket receive timeout when idle in milliseconds (defaults to :timeout value); :idle_interval - Ping connections after a period of inactivity in milliseconds. Defaults to 1000ms; :ssl - Set to true if ssl should be used (default: false ); :ssl_opts - A list of ssl options, see the tls_client_option from the ssl docs; :socket_options - Options to be given to the underlying socket (applies to both TCP and UNIX sockets); :target_server_type - Allows opening connections to a server in the given replica mode. The allowed values are :any , :primary and :secondary (default: :any ). If this option is used together with endpoints , we will traverse all endpoints until we find an endpoint matching the server type; :disconnect_on_error_codes - List of error code atoms that when encountered will disconnect the connection. This is useful when using Postgrex against systems that support failover, which when it occurs will emit certain error codes e.g. :read_only_sql_transaction (default: [] ); :show_sensitive_data_on_connection_error - By default, Postgrex hides all information during connection errors to avoid leaking credentials or other sensitive information. You can set this option if you wish to see complete errors and stacktraces during connection errors; The following options controls the pool and other Postgrex features: :prepare - How to prepare queries, either :named to use named queries or :unnamed to force unnamed queries (default: :named ); :transactions - Set to :strict to error on unexpected transaction state, otherwise set to :naive (default: :strict ); :pool - The pool module to use, defaults to DBConnection.ConnectionPool . See the pool documentation for more options. The default :pool_size for the default pool is 1. If you set a different pool, this option must be included with all requests contacting the pool; :types - The types module to use, see Postgrex.Types.define/3 , this option is only required when using custom encoding or decoding (default: Postgrex.DefaultTypes ); :search_path - A list of strings used to set the search path for the connection. This is useful when, for instance, an extension like citext is installed in a separate schema. If that schema is not in the connection's search path, Postgrex might not be able to recognize the extension's data type. When this option is nil , the search path is not modified. (default: nil ). See the PostgreSQL docs for more details. Postgrex uses the DBConnection library and supports all DBConnection options like :idle , :after_connect etc. See DBConnection.start_link/2 for more information. Examples iex&gt; { :ok , pid } = Postgrex . start_link ( database : &quot;postgres&quot; ) { :ok , # PID &lt; 0 . 69 . 0 &gt; } Run a query after connection has been established: iex&gt; { :ok , pid } = Postgrex . start_link ( after_connect : &amp; Postgrex . query! ( &amp;1 , &quot;SET TIME ZONE &#39;UTC&#39;;&quot; , [ ] ) ) { :ok , # PID &lt; 0 . 69 . 0 &gt; } Connect to postgres instance through a unix domain socket iex&gt; { :ok , pid } = Postgrex . start_link ( socket_dir : &quot;/tmp&quot; , database : &quot;postgres&quot; ) { :ok , # PID &lt; 0 . 69 . 0 &gt; } SSL client authentication When connecting to Postgres or CockroachDB instances over SSL it is idiomatic to use certificate authentication. Config files do not allowing passing functions, so use the init callback of the Ecto supervisor. In your Repository configuration: config :app , App.Repo , ssl : String . to_existing_atom ( System . get_env ( &quot;DB_SSL_ENABLED&quot; , &quot;true&quot; ) ) , verify_ssl : true And in App.Repo, set your :ssl_opts : def init ( _type , config ) do config = if config [ :verify_ssl ] do Keyword . put ( config , :ssl_opts , my_ssl_opts ( config [ :hostname ] ) ) else config end { :ok , config } end def my_ssl_opts ( server ) do [ verify : :verify_peer , cacertfile : System . get_env ( &quot;DB_CA_CERT_FILE&quot; ) , server_name_indication : String . to_charlist ( server ) , customize_hostname_check : [ match_fun : :public_key . pkix_verify_hostname_match_fun ( :https ) ] , depth : 3 ] end PgBouncer When using PgBouncer with transaction or statement pooling named prepared queries can not be used because the bouncer may route requests from the same postgrex connection to different PostgreSQL backend processes and discards named queries after the transactions closes. To force unnamed prepared queries set the :prepare option to :unnamed . Handling failover Some services, such as AWS Aurora, support failovers. The 2 options endpoints and target_server_type can be used together to achieve a faster fail-over. Imagine an AWS Aurora cluster named &quot;test&quot; with 2 instances. Use the following options minimize downtime by ensuring that Postgrex connects to the new primary instance as soon as possible. { :ok , pid } = Postgrex . start_link ( endpoints : [ { &quot;test.cluster-xyz.eu-west-1.rds.amazonaws.com&quot; , 5432 } , { &quot;test.cluster-ro-xyz.eu-west-1.rds.amazonaws.com&quot; , 5432 } ] , target_server_type : :primary , ( ... ) ) In the event of a fail-over, Postgrex gets first disconnected from what used to be the primary instance. The primary instance will then reboot and turn into a secondary instance. Meanwhile, one of the secondary instances will have turned into the new primary instance. However, the DNS entry of the primary endpoint provided by AWS can take some time to get updated. That is why it can be faster to let Postgrex iterate over all the instances of the cluster to find the new primary instance instead of waiting for the DNS update. If the cluster does not have DNS-backed primary and secondary endpoints (like the ones provided by AWS Aurora) or if the cluster is made of more than 2 instances, the hostname (and port) of all of the individual instances can be specified in the endpoints list: endpoints : [ { &quot;test-instance-1.xyz.eu-west-1.rds.amazonaws.com&quot; , 5432 } , { &quot;test-instance-2.xyz.eu-west-1.rds.amazonaws.com&quot; , 5432 } , ( ... ) , { &quot;test-instance-N.xyz.eu-west-1.rds.amazonaws.com&quot; , 5432 } ] Failover with SSL support As specified in Erlang :ssl.connect , host verification using :public_key.pkix_verify_hostname_match_fun(:https) requires that the ssl_opt server_name_indication is set, and for this reason, the aforementioned endpoints list can become a three element tuple as: endpoints : [ { &quot;test-instance-1.xyz.eu-west-1.rds.amazonaws.com&quot; , 5432 , [ ssl : [ server_name_indication : String . to_charlist ( &quot;test-instance-1.xyz.eu-west-1.rds.amazonaws.com&quot; ) ] ] } , ( ... ) , { &quot;test-instance-2.xyz.eu-west-1.rds.amazonaws.com&quot; , 5432 , [ ssl : [ server_name_indication : String . to_charlist ( &quot;test-instance-2.xyz.eu-west-1.rds.amazonaws.com&quot; ) ] ] } ]","ref":"Postgrex.html#start_link/1","title":"Postgrex.start_link/1","type":"function"},{"doc":"Returns a stream for a query on a connection. Stream consumes memory in chunks of at most max_rows rows (see Options). This is useful for processing large datasets. A stream must be wrapped in a transaction and may be used as an Enumerable or a Collectable . When used as an Enumerable with a COPY .. TO STDOUT SQL query no other queries or streams can be interspersed until the copy has finished. Otherwise it is possible to intersperse enumerable streams and queries. When used as a Collectable the values are passed as copy data with the query. No other queries or streams can be interspersed until the copy has finished. If the query is not copying to the database the copy data will still be sent but is silently discarded. Options :max_rows - Maximum numbers of rows in a result (default to 500 ) :decode_mapper - Fun to map each row in the result to a term after decoding, (default: fn x -&gt; x end ); :mode - set to :savepoint to use a savepoint to rollback to before an execute on error, otherwise set to :transaction (default: :transaction ); Examples Postgrex . transaction ( pid , fn ( conn ) -&gt; query = Postgrex . prepare! ( conn , &quot;&quot; , &quot;COPY posts TO STDOUT&quot; ) stream = Postgrex . stream ( conn , query , [ ] ) result_to_iodata = fn ( % Postgrex.Result { rows : rows } ) -&gt; rows end Enum . into ( stream , File . stream! ( &quot;posts&quot; ) , result_to_iodata ) end ) Postgrex . transaction ( pid , fn ( conn ) -&gt; stream = Postgrex . stream ( conn , &quot;COPY posts FROM STDIN&quot; , [ ] ) Enum . into ( File . stream! ( &quot;posts&quot; ) , stream ) end )","ref":"Postgrex.html#stream/4","title":"Postgrex.stream/4","type":"function"},{"doc":"Acquire a lock on a connection and run a series of requests inside a transaction. The result of the transaction fun is return inside an :ok tuple: {:ok, result} . To use the locked connection call the request with the connection reference passed as the single argument to the fun . If the connection disconnects all future calls using that connection reference will fail. rollback/2 rolls back the transaction and causes the function to return {:error, reason} . transaction/3 can be nested multiple times if the connection reference is used to start a nested transaction. The top level transaction function is the actual transaction. Options :queue - Whether to wait for connection in a queue (default: true ); :timeout - Transaction timeout (default: 15000 ); :mode - Set to :savepoint to use savepoints instead of an SQL transaction, otherwise set to :transaction (default: :transaction ); The :timeout is for the duration of the transaction and all nested transactions and requests. This timeout overrides timeouts set by internal transactions and requests. The :mode will be used for all requests inside the transaction function. Example { :ok , res } = Postgrex . transaction ( pid , fn ( conn ) -&gt; Postgrex . query! ( conn , &quot;SELECT title FROM posts&quot; , [ ] ) end )","ref":"Postgrex.html#transaction/3","title":"Postgrex.transaction/3","type":"function"},{"doc":"A connection process name, pid or reference. A connection reference is used when making multiple requests to the same connection, see transaction/3 .","ref":"Postgrex.html#t:conn/0","title":"Postgrex.conn/0","type":"type"},{"doc":"","ref":"Postgrex.html#t:execute_option/0","title":"Postgrex.execute_option/0","type":"type"},{"doc":"","ref":"Postgrex.html#t:option/0","title":"Postgrex.option/0","type":"type"},{"doc":"","ref":"Postgrex.html#t:start_option/0","title":"Postgrex.start_option/0","type":"type"},{"doc":"","ref":"Postgrex.Extensions.Multirange.html","title":"Postgrex.Extensions.Multirange","type":"module"},{"doc":"Callback implementation for c:Postgrex.SuperExtension.decode/1 .","ref":"Postgrex.Extensions.Multirange.html#decode/1","title":"Postgrex.Extensions.Multirange.decode/1","type":"function"},{"doc":"","ref":"Postgrex.Extensions.Multirange.html#decode/3","title":"Postgrex.Extensions.Multirange.decode/3","type":"function"},{"doc":"Callback implementation for c:Postgrex.SuperExtension.encode/1 .","ref":"Postgrex.Extensions.Multirange.html#encode/1","title":"Postgrex.Extensions.Multirange.encode/1","type":"function"},{"doc":"","ref":"Postgrex.Extensions.Multirange.html#encode/2","title":"Postgrex.Extensions.Multirange.encode/2","type":"function"},{"doc":"Callback implementation for c:Postgrex.SuperExtension.format/1 .","ref":"Postgrex.Extensions.Multirange.html#format/1","title":"Postgrex.Extensions.Multirange.format/1","type":"function"},{"doc":"Callback implementation for c:Postgrex.SuperExtension.init/1 .","ref":"Postgrex.Extensions.Multirange.html#init/1","title":"Postgrex.Extensions.Multirange.init/1","type":"function"},{"doc":"Callback implementation for c:Postgrex.SuperExtension.matching/1 .","ref":"Postgrex.Extensions.Multirange.html#matching/1","title":"Postgrex.Extensions.Multirange.matching/1","type":"function"},{"doc":"Callback implementation for c:Postgrex.SuperExtension.oids/2 .","ref":"Postgrex.Extensions.Multirange.html#oids/2","title":"Postgrex.Extensions.Multirange.oids/2","type":"function"},{"doc":"Struct for PostgreSQL multirange . Fields ranges","ref":"Postgrex.Multirange.html","title":"Postgrex.Multirange","type":"module"},{"doc":"","ref":"Postgrex.Multirange.html#t:t/0","title":"Postgrex.Multirange.t/0","type":"type"},{"doc":"API for notifications (pub/sub) in PostgreSQL. In order to use it, first you need to start the notification process. In your supervision tree: { Postgrex.Notifications , name : MyApp.Notifications } Then you can listen to certain channels: { :ok , listen_ref } = Postgrex.Notifications . listen ( MyApp.Notifications , &quot;channel&quot; ) Now every time a message is broadcast on said channel, for example via PostgreSQL command line: NOTIFY &quot;channel&quot; , &quot;Oh hai!&quot; ; You will receive a message in the format: { :notification , notification_pid , listen_ref , channel , message } Async connect, auto-reconnects and missed notifications By default, the notification system establishes a connection to the database on initialization, you can configure the connection to happen asynchronously. You can also configure the connection to automatically reconnect. Note however that when the notification system is waiting for a connection, any notifications that occur during the disconnection period are not queued and cannot be recovered. Similarly, any listen command will be queued until the connection is up. There is a race condition between starting to listen and notifications being issued &quot;at the same time&quot;, as explained (in the PostgreSQL documentation)[ https://www.postgresql.org/docs/current/sql-listen.html] . If your application needs to keep a consistent representation of data, follow the three-step approach of first subscribing, then obtaining the current state of data, then handling the incoming notifications. Beware that the same race condition applies to auto-reconnects. A simple way of dealing with this issue is not using the auto-reconnect feature directly, but monitoring and re-starting the Notifications process, then subscribing to channel messages over again, using the same three-step approach. A note on casing While PostgreSQL seems to behave as case-insensitive, it actually has a very peculiar behaviour on casing. When you write: SELECT * FROM POSTS PostgreSQL actually converts POSTS into the lowercase posts . That's why both SELECT * FROM POSTS and SELECT * FROM posts feel equivalent. However, if you wrap the table name in quotes, then the casing in quotes will be preserved. These same rules apply to PostgreSQL notification channels. More importantly, whenever Postgrex.Notifications listens to a channel, it wraps the channel name in quotes. Therefore, if you listen to a channel named &quot;fooBar&quot; and you send a notification without quotes in the channel name, such as: NOTIFY fooBar , &quot;Oh hai!&quot; ; The notification will not be received by Postgrex.Notifications because the notification will be effectively sent to &quot;foobar&quot; and not &quot;fooBar&quot; . Therefore, you must guarantee one of the two following properties: If you can wrap the channel name in quotes when sending a notification, then make sure the channel name has the exact same casing when listening and sending notifications If you cannot wrap the channel name in quotes when sending a notification, then make sure to give the lowercased channel name when listening","ref":"Postgrex.Notifications.html","title":"Postgrex.Notifications","type":"module"},{"doc":"Listens to an asynchronous notification channel using the LISTEN command. A message {:notification, connection_pid, ref, channel, payload} will be sent to the calling process when a notification is received. It returns {:ok, reference} . It may also return {:eventually, reference} if the notification process is not currently connected to the database and it was started with :sync_connect set to false or :auto_reconnect set to true. The reference can be used to issue an unlisten/3 command. Options :timeout - Call timeout (default: 5000 )","ref":"Postgrex.Notifications.html#listen/3","title":"Postgrex.Notifications.listen/3","type":"function"},{"doc":"Listens to an asynchronous notification channel channel . See listen/2 .","ref":"Postgrex.Notifications.html#listen!/3","title":"Postgrex.Notifications.listen!/3","type":"function"},{"doc":"Start the notification connection process and connect to postgres. The options that this function accepts are the same as those accepted by Postgrex.start_link/1 , as well as the extra options :sync_connect , :auto_reconnect , :reconnect_backoff , and :configure . Options :sync_connect - controls if the connection should be established on boot or asynchronously right after boot. Defaults to true . :auto_reconnect - automatically attempt to reconnect to the database in event of a disconnection. See the note about async connect and auto-reconnects above. Defaults to false , which means the process terminates. :reconnect_backoff - time (in ms) between reconnection attempts when auto_reconnect is enabled. Defaults to 500 . :idle_interval - while also accepted on Postgrex.start_link/1 , it has a default of 5000ms in Postgrex.Notifications (instead of 1000ms). :configure - A function to run before every connect attempt to dynamically configure the options as a {module, function, args} , where the current options will prepended to args . Defaults to nil .","ref":"Postgrex.Notifications.html#start_link/1","title":"Postgrex.Notifications.start_link/1","type":"function"},{"doc":"Stops listening on the given channel by passing the reference returned from listen/2 . Options :timeout - Call timeout (default: 5000 )","ref":"Postgrex.Notifications.html#unlisten/3","title":"Postgrex.Notifications.unlisten/3","type":"function"},{"doc":"Stops listening on the given channel by passing the reference returned from listen/2 .","ref":"Postgrex.Notifications.html#unlisten!/3","title":"Postgrex.Notifications.unlisten!/3","type":"function"},{"doc":"","ref":"Postgrex.Notifications.html#t:server/0","title":"Postgrex.Notifications.server/0","type":"type"},{"doc":"Query struct returned from a successfully prepared query. Its public fields are: name - The name of the prepared statement; statement - The prepared statement; columns - The column names; ref - A reference used to identify prepared queries; Prepared queries Once a query is prepared with Postgrex.prepare/4 , the returned query will have its ref field set to a reference. When Postgrex.execute/4 is called with the prepared query, it always returns a query. If the ref field in the query given to execute and the one returned are the same, it means the cached prepared query was used. If the ref field is not the same, it means the query had to be re-prepared.","ref":"Postgrex.Query.html","title":"Postgrex.Query","type":"module"},{"doc":"","ref":"Postgrex.Query.html#t:t/0","title":"Postgrex.Query.t/0","type":"type"},{"doc":"A process that receives and sends PostgreSQL replication messages. Note: this module is experimental and may be subject to changes in the future. Logical replication Let's see how to use this module for connecting to PostgreSQL for logical replication. First of all, you need to configure the wal level in PostgreSQL to logical. Run this inside your PostgreSQL shell/configuration: ALTER SYSTEM SET wal_level = &#39;logical&#39; ; ALTER SYSTEM SET max_wal_senders = &#39;10&#39; ; ALTER SYSTEM SET max_replication_slots = &#39;10&#39; ; Then you must restart your server . Alternatively, you can set those values when starting &quot;postgres&quot;. This is useful, for example, when running it from Docker: services : postgres : image : postgres : 14 env : ... command : [ &quot;postgres&quot; , &quot;-c&quot; , &quot;wal_level=logical&quot; ] For CI, GitHub Actions do not support setting command, so you can update and restart Postgres instead in a step: - name : &quot;Set PG settings&quot; run : | docker exec $ { { job . services . postgres . id } } sh - c &#39;echo &quot;wal_level=logical&quot; &gt;&gt; /var/lib/postgresql/data/postgresql.conf&#39; docker restart $ { { job . services . pg . id } } Then you must create a publication to be replicated. This can be done in any session: CREATE PUBLICATION postgrex_example FOR ALL TABLES ; You can also filter if you want to publish insert, update, delete or a subset of them: # Skips updates (keeps inserts, deletes, begins, commits, etc) create PUBLICATION postgrex_example FOR ALL TABLES WITH ( publish = &#39;insert,delete&#39; ) ; # Skips inserts, updates, and deletes (keeps begins, commits, etc) create PUBLICATION postgrex_example FOR ALL TABLES WITH ( publish = &#39;&#39; ) ; Now we are ready to create module that starts a replication slot and listens to our publication. Our example will use the pgoutput for logical replication and print all incoming messages to the terminal: Mix . install ( [ :postgrex ] ) defmodule Repl do use Postgrex.ReplicationConnection def start_link ( opts ) do # Automatically reconnect if we lose connection. extra_opts = [ auto_reconnect : true ] Postgrex.ReplicationConnection . start_link ( __MODULE__ , :ok , extra_opts ++ opts ) end @impl true def init ( :ok ) do { :ok , %{ step : :disconnected } } end @impl true def handle_connect ( state ) do query = &quot;CREATE_REPLICATION_SLOT postgrex TEMPORARY LOGICAL pgoutput NOEXPORT_SNAPSHOT&quot; { :query , query , %{ state | step : :create_slot } } end @impl true def handle_result ( results , %{ step : :create_slot } = state ) when is_list ( results ) do query = &quot;START_REPLICATION SLOT postgrex LOGICAL 0/0 (proto_version &#39;1&#39;, publication_names &#39;postgrex_example&#39;)&quot; { :stream , query , [ ] , %{ state | step : :streaming } } end @impl true # https://www.postgresql.org/docs/14/protocol-replication.html def handle_data ( &lt;&lt; ?w , _wal_start :: 64 , _wal_end :: 64 , _clock :: 64 , rest :: binary &gt;&gt; , state ) do IO . inspect ( rest ) { :noreply , state } end def handle_data ( &lt;&lt; ?k , wal_end :: 64 , _clock :: 64 , reply &gt;&gt; , state ) do messages = case reply do 1 -&gt; [ &lt;&lt; ?r , wal_end + 1 :: 64 , wal_end + 1 :: 64 , wal_end + 1 :: 64 , current_time ( ) :: 64 , 0 &gt;&gt; ] 0 -&gt; [ ] end { :noreply , messages , state } end @epoch DateTime . to_unix ( ~U[2000-01-01 00:00:00Z] , :microsecond ) defp current_time ( ) , do : System . os_time ( :microsecond ) - @epoch end { :ok , pid } = Repl . start_link ( host : &quot;localhost&quot; , database : &quot;demo_dev&quot; , username : &quot;postgres&quot; , ) Process . sleep ( :infinity ) use options use Postgrex.ReplicationConnection accepts a list of options which configures the child specification and therefore how it runs under a supervisor. The generated child_spec/1 can be customized with the following options: :id - the child specification identifier, defaults to the current module :restart - when the child should be restarted, defaults to :permanent :shutdown - how to shut down the child, either immediately or by giving it time to shut down For example: use Postgrex.ReplicationConnection , restart : :transient , shutdown : 10_000 See the &quot;Child specification&quot; section in the Supervisor module for more detailed information. The @doc annotation immediately preceding use Postgrex.ReplicationConnection will be attached to the generated child_spec/1 function. Name registration A Postgrex.ReplicationConnection is bound to the same name registration rules as a GenServer . Read more about them in the GenServer docs.","ref":"Postgrex.ReplicationConnection.html","title":"Postgrex.ReplicationConnection","type":"behaviour"},{"doc":"Calls the given replication server.","ref":"Postgrex.ReplicationConnection.html#call/3","title":"Postgrex.ReplicationConnection.call/3","type":"function"},{"doc":"Returns the integer representation of an LSN value, given its string representation. It returns :error if the provided string is not a valid LSN. Log Sequence Numbers PostgreSQL uses two representations for the Log Sequence Number (LSN): An unsigned 64-bit integer. Used internally by PostgreSQL and sent in the XLogData replication messages. A string of two hexadecimal numbers of up to eight digits each, separated by a slash. e.g. 1/F73E0220 . This is the form accepted by start_replication/2 . For more information on Log Sequence Numbers, see PostgreSQL pg_lsn docs and PostgreSQL WAL internals docs .","ref":"Postgrex.ReplicationConnection.html#decode_lsn/1","title":"Postgrex.ReplicationConnection.decode_lsn/1","type":"function"},{"doc":"Returns the string representation of an LSN value, given its integer representation. It returns :error if the provided integer falls outside the range for a valid unsigned 64-bit integer. Log Sequence Numbers PostgreSQL uses two representations for the Log Sequence Number (LSN): An unsigned 64-bit integer. Used internally by PostgreSQL and sent in the XLogData replication messages. A string of two hexadecimal numbers of up to eight digits each, separated by a slash. e.g. 1/F73E0220 . This is the form accepted by start_replication/2 . For more information on Log Sequence Numbers, see PostgreSQL pg_lsn docs and PostgreSQL WAL internals docs .","ref":"Postgrex.ReplicationConnection.html#encode_lsn/1","title":"Postgrex.ReplicationConnection.encode_lsn/1","type":"function"},{"doc":"Callback for call/3 . Replies must be sent with reply/2 . If auto_reconnect: false (the default) and there is a disconnection, the process will terminate and the caller will exit even if no reply is sent. However, if auto_reconnect is set to true, a disconnection will keep the process alive, which means that any command that has not yet been replied to should eventually do so. One simple approach is to reply to any pending commands on handle_disconnect/1 .","ref":"Postgrex.ReplicationConnection.html#c:handle_call/3","title":"Postgrex.ReplicationConnection.handle_call/3","type":"callback"},{"doc":"Invoked after connecting. This may be invoked multiple times if :auto_reconnect is set to true.","ref":"Postgrex.ReplicationConnection.html#c:handle_connect/1","title":"Postgrex.ReplicationConnection.handle_connect/1","type":"callback"},{"doc":"Callback for :stream outputs. If any callback returns {:stream, iodata, opts, state} , then this callback will be eventually called with the result of the query. It receives binary streaming messages. This can be useful for replication and copy commands. For replication, the format of the messages are described under the START_REPLICATION section in PostgreSQL docs . Replication messages may require explicit acknowledgement, which can be done by returning a list of binaries according to the replication protocol.","ref":"Postgrex.ReplicationConnection.html#c:handle_data/2","title":"Postgrex.ReplicationConnection.handle_data/2","type":"callback"},{"doc":"Invoked after disconnecting. This is only invoked if :auto_reconnect is set to true.","ref":"Postgrex.ReplicationConnection.html#c:handle_disconnect/1","title":"Postgrex.ReplicationConnection.handle_disconnect/1","type":"callback"},{"doc":"Callback for Kernel.send/2 .","ref":"Postgrex.ReplicationConnection.html#c:handle_info/2","title":"Postgrex.ReplicationConnection.handle_info/2","type":"callback"},{"doc":"Callback for :query outputs. If any callback returns {:query, iodata, state} , then this callback will be immediately called with the result of the query. Please note that even though replication connections use the simple query protocol, Postgres currently limits them to single command queries. Due to this constraint, this callback will be passed either a list with a single successful query result or an error.","ref":"Postgrex.ReplicationConnection.html#c:handle_result/2","title":"Postgrex.ReplicationConnection.handle_result/2","type":"callback"},{"doc":"Callback for process initialization. This is called once and before the Postgrex connection is established.","ref":"Postgrex.ReplicationConnection.html#c:init/1","title":"Postgrex.ReplicationConnection.init/1","type":"callback"},{"doc":"Replies to the given call/3 .","ref":"Postgrex.ReplicationConnection.html#reply/2","title":"Postgrex.ReplicationConnection.reply/2","type":"function"},{"doc":"Starts a replication process with the given callback module . Options The options that this function accepts are the same as those accepted by Postgrex.start_link/1 , except for :idle_interval . It also accepts extra options for connection management, documented below. Also note this function also automatically set :replication to &quot;database&quot; as part of the connection :parameters if none is set yet. Connection options :sync_connect - controls if the connection should be established on boot or asynchronously right after boot. Defaults to true . :auto_reconnect - automatically attempt to reconnect to the database in event of a disconnection. See the note about async connect and auto-reconnects above. Defaults to false , which means the process terminates. :reconnect_backoff - time (in ms) between reconnection attempts when :auto_reconnect is enabled. Defaults to 500 .","ref":"Postgrex.ReplicationConnection.html#start_link/3","title":"Postgrex.ReplicationConnection.start_link/3","type":"function"},{"doc":"","ref":"Postgrex.ReplicationConnection.html#t:ack/0","title":"Postgrex.ReplicationConnection.ack/0","type":"type"},{"doc":"","ref":"Postgrex.ReplicationConnection.html#t:query/0","title":"Postgrex.ReplicationConnection.query/0","type":"type"},{"doc":"","ref":"Postgrex.ReplicationConnection.html#t:server/0","title":"Postgrex.ReplicationConnection.server/0","type":"type"},{"doc":"","ref":"Postgrex.ReplicationConnection.html#t:state/0","title":"Postgrex.ReplicationConnection.state/0","type":"type"},{"doc":"The following options configure streaming: :max_messages - The maximum number of replications messages that can be accumulated from the wire until they are relayed to handle_data/2 . Defaults to 500 .","ref":"Postgrex.ReplicationConnection.html#t:stream_opts/0","title":"Postgrex.ReplicationConnection.stream_opts/0","type":"type"},{"doc":"Result struct returned from any successful query. Its fields are: command - An atom or a list of atoms of the query command, for example: :select , :insert , or [:rollback, :release] ; columns - The column names; rows - The result set. A list of lists, each inner list corresponding to a row, each element in the inner list corresponds to a column; num_rows - The number of fetched or affected rows; connection_id - The OS pid of the PostgreSQL backend that executed the query; messages - A list of maps of messages, such as hints and notices, sent by the the driver during the execution of the query","ref":"Postgrex.Result.html","title":"Postgrex.Result","type":"module"},{"doc":"","ref":"Postgrex.Result.html#t:t/0","title":"Postgrex.Result.t/0","type":"type"},{"doc":"A generic connection suitable for simple queries and pubsub functionality. On its own, a SimpleConnection server only maintains a connection. To execute queries, process results, or relay notices you must implement a callback module with the SimpleConnection behaviour. Example The SimpleConnection behaviour abstracts common client/server interactions, along with optional mechanisms for running queries or relaying notifications. Let's start with a minimal callback module that executes a query and relays the result back to the caller. defmodule MyConnection do @behaviour Postgrex.SimpleConnection @impl true def init ( _args ) do { :ok , %{ from : nil } } end @impl true def handle_call ( { :query , query } , from , state ) do { :query , query , %{ state | from : from } } end @impl true def handle_result ( results , state ) when is_list ( results ) do SimpleConnection . reply ( state . from , results ) { :noreply , state } end @impl true def handle_result ( % Postgrex.Error { } = error , state ) do SimpleConnection . reply ( state . from , error ) { :noreply , state } end end # Start the connection { :ok , pid } = SimpleConnection . start_link ( MyConnection , [ ] , database : &quot;demo&quot; ) # Execute a literal query SimpleConnection . call ( pid , { :query , &quot;SELECT 1&quot; } ) # =&gt; %Postgrex.Result{rows: [[&quot;1&quot;]]} We start a connection by passing the callback module, callback options, and server options to SimpleConnection.start_link/3 . The init/1 function receives any callback options and returns the callback state. Queries are sent through SimpleConnection.call/2 , executed on the server, and the result is handed off to handle_result/2 . At that point the callback can process the result before replying back to the caller with SimpleConnection.reply/2 . Building a PubSub Connection With the notify/3 callback you can also build a pubsub server on top of LISTEN/NOTIFY . Here's a naive pubsub implementation: defmodule MyPubSub do @behaviour Postgrex.SimpleConnection defstruct [ :from , listeners : %{ } ] @impl true def init ( args ) do { :ok , struct! ( __MODULE__ , args ) } end @impl true def notify ( channel , payload , state ) do for pid &lt;- state . listeners [ channel ] do send ( pid , { :notice , channel , payload } ) end end @impl true def handle_call ( { :listen , channel } , { pid , _ } = from , state ) do listeners = Map . update ( state . listeners , channel , [ pid ] , &amp; [ pid | &amp;1 ] ) { :query , ~s(LISTEN &quot; \#{ channel } &quot;) , %{ state | from : from , listeners : listeners } } end def handle_call ( { :query , query } , from , state ) do { :query , query , %{ state | from : from } } end @impl true def handle_result ( _results , state ) do SimpleConnection . reply ( state . from , :ok ) { :noreply , %{ state | from : nil } } end end # Start the connection { :ok , pid } = SimpleConnection . start_link ( MyPubSub , [ ] , database : &quot;demo&quot; ) # Start listening to the &quot;demo&quot; channel SimpleConnection . call ( pid , { :listen , &quot;demo&quot; } ) # =&gt; %Postgrex.Result{command: :listen} # Notify all listeners SimpleConnection . call ( pid , { :query , ~s(NOTIFY &quot;demo&quot;, &#39;hello&#39;) } ) # =&gt; %Postgrex.Result{command: :notify} # Check the inbox to see the notice message flush ( ) # =&gt; {:notice, &quot;demo&quot;, &quot;hello&quot;} See Postgrex.Notifications for a more complex implementation that can unlisten, handle process exits, and persist across reconnection. Name registration A Postgrex.SimpleConnection is bound to the same name registration rules as a GenServer . Read more about them in the GenServer docs.","ref":"Postgrex.SimpleConnection.html","title":"Postgrex.SimpleConnection","type":"behaviour"},{"doc":"Calls the given server. Wrapper for :gen_statem.call/3 .","ref":"Postgrex.SimpleConnection.html#call/3","title":"Postgrex.SimpleConnection.call/3","type":"function"},{"doc":"Callback for SimpleConnection.call/3 . Replies must be sent with SimpleConnection.reply/2 .","ref":"Postgrex.SimpleConnection.html#c:handle_call/3","title":"Postgrex.SimpleConnection.handle_call/3","type":"callback"},{"doc":"Invoked after connecting or reconnecting. This may be called multiple times if :auto_reconnect is true.","ref":"Postgrex.SimpleConnection.html#c:handle_connect/1","title":"Postgrex.SimpleConnection.handle_connect/1","type":"callback"},{"doc":"Invoked after disconnection. This is invoked regardless of the :auto_reconnect option.","ref":"Postgrex.SimpleConnection.html#c:handle_disconnect/1","title":"Postgrex.SimpleConnection.handle_disconnect/1","type":"callback"},{"doc":"Callback for Kernel.send/2 .","ref":"Postgrex.SimpleConnection.html#c:handle_info/2","title":"Postgrex.SimpleConnection.handle_info/2","type":"callback"},{"doc":"Callback for processing or relaying queries executed via {:query, query, state} . Either a list of successful query results or an error will be passed to this callback. A list is passed because the simple query protocol allows multiple commands to be issued in a single query.","ref":"Postgrex.SimpleConnection.html#c:handle_result/2","title":"Postgrex.SimpleConnection.handle_result/2","type":"callback"},{"doc":"Callback for process initialization. This is called once and before the Postgrex connection is established.","ref":"Postgrex.SimpleConnection.html#c:init/1","title":"Postgrex.SimpleConnection.init/1","type":"callback"},{"doc":"Callback for processing or relaying pubsub notifications.","ref":"Postgrex.SimpleConnection.html#c:notify/3","title":"Postgrex.SimpleConnection.notify/3","type":"callback"},{"doc":"Replies to the given client. Wrapper for :gen_statem.reply/2 .","ref":"Postgrex.SimpleConnection.html#reply/2","title":"Postgrex.SimpleConnection.reply/2","type":"function"},{"doc":"Start the connection process and connect to Postgres. The options that this function accepts are the same as those accepted by Postgrex.start_link/1 , as well as the extra options :sync_connect , :auto_reconnect , :reconnect_backoff , and :configure . Options :auto_reconnect - automatically attempt to reconnect to the database in event of a disconnection. See the note about async connect and auto-reconnects above. Defaults to false , which means the process terminates. :configure - A function to run before every connect attempt to dynamically configure the options as a {module, function, args} , where the current options will prepended to args . Defaults to nil . :idle_interval - while also accepted on Postgrex.start_link/1 , it has a default of 5000ms in Postgrex.SimpleConnection (instead of 1000ms). :reconnect_backoff - time (in ms) between reconnection attempts when auto_reconnect is enabled. Defaults to 500 . :sync_connect - controls if the connection should be established on boot or asynchronously right after boot. Defaults to true .","ref":"Postgrex.SimpleConnection.html#start_link/3","title":"Postgrex.SimpleConnection.start_link/3","type":"function"},{"doc":"","ref":"Postgrex.SimpleConnection.html#t:from/0","title":"Postgrex.SimpleConnection.from/0","type":"type"},{"doc":"","ref":"Postgrex.SimpleConnection.html#t:query/0","title":"Postgrex.SimpleConnection.query/0","type":"type"},{"doc":"","ref":"Postgrex.SimpleConnection.html#t:state/0","title":"Postgrex.SimpleConnection.state/0","type":"type"},{"doc":"Stream struct returned from stream commands. All of its fields are private.","ref":"Postgrex.Stream.html","title":"Postgrex.Stream","type":"module"},{"doc":"","ref":"Postgrex.Stream.html#t:t/0","title":"Postgrex.Stream.t/0","type":"type"},{"doc":"Struct for PostgreSQL box . Fields upper_right bottom_left","ref":"Postgrex.Box.html","title":"Postgrex.Box","type":"module"},{"doc":"","ref":"Postgrex.Box.html#t:t/0","title":"Postgrex.Box.t/0","type":"type"},{"doc":"Struct for PostgreSQL circle . Fields center radius","ref":"Postgrex.Circle.html","title":"Postgrex.Circle","type":"module"},{"doc":"","ref":"Postgrex.Circle.html#t:t/0","title":"Postgrex.Circle.t/0","type":"type"},{"doc":"Struct for PostgreSQL inet / cidr . Fields address netmask","ref":"Postgrex.INET.html","title":"Postgrex.INET","type":"module"},{"doc":"","ref":"Postgrex.INET.html#t:t/0","title":"Postgrex.INET.t/0","type":"type"},{"doc":"Struct for PostgreSQL interval . Fields months days secs microsecs","ref":"Postgrex.Interval.html","title":"Postgrex.Interval","type":"module"},{"doc":"","ref":"Postgrex.Interval.html#compare/2","title":"Postgrex.Interval.compare/2","type":"function"},{"doc":"","ref":"Postgrex.Interval.html#to_string/1","title":"Postgrex.Interval.to_string/1","type":"function"},{"doc":"","ref":"Postgrex.Interval.html#t:t/0","title":"Postgrex.Interval.t/0","type":"type"},{"doc":"Struct for PostgreSQL lexeme . Fields word positions","ref":"Postgrex.Lexeme.html","title":"Postgrex.Lexeme","type":"module"},{"doc":"","ref":"Postgrex.Lexeme.html#t:t/0","title":"Postgrex.Lexeme.t/0","type":"type"},{"doc":"Struct for PostgreSQL line . Note, lines are stored in PostgreSQL in the form {a, b, c} , which parameterizes a line as a*x + b*y + c = 0 . Fields a b c","ref":"Postgrex.Line.html","title":"Postgrex.Line","type":"module"},{"doc":"","ref":"Postgrex.Line.html#t:t/0","title":"Postgrex.Line.t/0","type":"type"},{"doc":"Struct for PostgreSQL lseg . Fields point1 point2","ref":"Postgrex.LineSegment.html","title":"Postgrex.LineSegment","type":"module"},{"doc":"","ref":"Postgrex.LineSegment.html#t:t/0","title":"Postgrex.LineSegment.t/0","type":"type"},{"doc":"Struct for PostgreSQL macaddr . Fields address","ref":"Postgrex.MACADDR.html","title":"Postgrex.MACADDR","type":"module"},{"doc":"","ref":"Postgrex.MACADDR.html#t:macaddr/0","title":"Postgrex.MACADDR.macaddr/0","type":"type"},{"doc":"","ref":"Postgrex.MACADDR.html#t:t/0","title":"Postgrex.MACADDR.t/0","type":"type"},{"doc":"Struct for PostgreSQL path . Fields open points","ref":"Postgrex.Path.html","title":"Postgrex.Path","type":"module"},{"doc":"","ref":"Postgrex.Path.html#t:t/0","title":"Postgrex.Path.t/0","type":"type"},{"doc":"Struct for PostgreSQL point . Fields x y","ref":"Postgrex.Point.html","title":"Postgrex.Point","type":"module"},{"doc":"","ref":"Postgrex.Point.html#t:t/0","title":"Postgrex.Point.t/0","type":"type"},{"doc":"Struct for PostgreSQL polygon . Fields vertices","ref":"Postgrex.Polygon.html","title":"Postgrex.Polygon","type":"module"},{"doc":"","ref":"Postgrex.Polygon.html#t:t/0","title":"Postgrex.Polygon.t/0","type":"type"},{"doc":"Struct for PostgreSQL range . Fields lower upper lower_inclusive upper_inclusive","ref":"Postgrex.Range.html","title":"Postgrex.Range","type":"module"},{"doc":"","ref":"Postgrex.Range.html#t:t/0","title":"Postgrex.Range.t/0","type":"type"},{"doc":"The default module used to encode/decode PostgreSQL types. Type modules are given as the :types option in Postgrex.start_link/1 .","ref":"Postgrex.DefaultTypes.html","title":"Postgrex.DefaultTypes","type":"module"},{"doc":"An extension knows how to encode and decode PostgreSQL types to and from Elixir values. Custom extensions can be enabled via Postgrex.Types.define/3 . Postgrex.Types.define/3 must be called on its own file, outside of any module and function, as it only needs to be defined once during compilation. For example to support label trees using the text encoding format: defmodule MyApp.LTree do @behaviour Postgrex.Extension # It can be memory efficient to copy the decoded binary because a # reference counted binary that points to a larger binary will be passed # to the decode/4 callback. Copying the binary can allow the larger # binary to be garbage collected sooner if the copy is going to be kept # for a longer period of time. See `:binary.copy/1` for more # information. def init ( opts ) do Keyword . get ( opts , :decode_copy , :copy ) end # Use this extension when `type` from %Postgrex.TypeInfo{} is &quot;ltree&quot; def matching ( _state ) , do : [ type : &quot;ltree&quot; ] # Use the text format, &quot;ltree&quot; does not have a binary format. def format ( _state ) , do : :text # Use quoted expression to encode a string that is the same as # postgresql&#39;s ltree text format. The quoted expression should contain # clauses that match those of a `case` or `fn`. Encoding matches on the # value and returns encoded `iodata()`. The first 4 bytes in the # `iodata()` must be the byte size of the rest of the encoded data, as a # signed 32bit big endian integer. def encode ( _state ) do quote do bin when is_binary ( bin ) -&gt; [ &lt;&lt; byte_size ( bin ) :: signed - size ( 32 ) &gt;&gt; | bin ] end end # Use quoted expression to decode the data to a string. Decoding matches # on an encoded binary with the same signed 32bit big endian integer # length header. def decode ( :reference ) do quote do &lt;&lt; len :: signed - size ( 32 ) , bin :: binary - size ( len ) &gt;&gt; -&gt; bin end end def decode ( :copy ) do quote do &lt;&lt; len :: signed - size ( 32 ) , bin :: binary - size ( len ) &gt;&gt; -&gt; :binary . copy ( bin ) end end end This example could be used in a custom types module: Postgrex.Types . define ( MyApp.Types , [ MyApp.LTree ] ) Or pass in opts for the extension that will be passed to the init/1 callback: Postgrex.Types . define ( MyApp.Types , [ { MyApp.LTree , [ decode_copy : :copy ] } ] )","ref":"Postgrex.Extension.html","title":"Postgrex.Extension","type":"behaviour"},{"doc":"Returns a quoted list of clauses that decode a binary to an Elixir value. The pattern must use binary syntax and decode a fixed length using the signed 32 bit big endian integer byte length header. def decode ( _ ) do quote do # length header is in bytes &lt;&lt; len :: signed - 32 , integer :: signed - size ( len ) - unit ( 8 ) &gt;&gt; -&gt; integer end end","ref":"Postgrex.Extension.html#c:decode/1","title":"Postgrex.Extension.decode/1","type":"callback"},{"doc":"Returns a quoted list of clauses that encode an Elixir value to iodata. It must use a signed 32 bit big endian integer byte length header. def encode ( _ ) do quote do integer -&gt; &lt;&lt; 8 :: signed - 32 , integer :: signed - 64 &gt;&gt; end end","ref":"Postgrex.Extension.html#c:encode/1","title":"Postgrex.Extension.encode/1","type":"callback"},{"doc":"Returns the format the type should be encoded as. See http://www.postgresql.org/docs/9.4/static/protocol-overview.html#PROTOCOL-FORMAT-CODES .","ref":"Postgrex.Extension.html#c:format/1","title":"Postgrex.Extension.format/1","type":"callback"},{"doc":"Should perform any initialization of the extension. The function receives the user options. The state returned from this function will be passed to other callbacks.","ref":"Postgrex.Extension.html#c:init/1","title":"Postgrex.Extension.init/1","type":"callback"},{"doc":"Specifies the types the extension matches, see Postgrex.TypeInfo for specification of the fields.","ref":"Postgrex.Extension.html#c:matching/1","title":"Postgrex.Extension.matching/1","type":"callback"},{"doc":"Prelude defines properties and values that are attached to the body of the types module.","ref":"Postgrex.Extension.html#c:prelude/1","title":"Postgrex.Extension.prelude/1","type":"callback"},{"doc":"","ref":"Postgrex.Extension.html#t:state/0","title":"Postgrex.Extension.state/0","type":"type"},{"doc":"","ref":"Postgrex.Extension.html#t:t/0","title":"Postgrex.Extension.t/0","type":"type"},{"doc":"The information about a type that is provided to the custom encoder/decoder functions. See http://www.postgresql.org/docs/9.4/static/catalog-pg-type.html for clarifications of the fields. oid - The type's id; type - The type name; send - The name of the &quot;send&quot; function (the function postgres uses to convert the type to its binary format); receive - The name of the &quot;receive&quot; function (the function postgres uses to convert the type from its binary format); output - The name of the &quot;output&quot; function (the function postgres uses to convert the type to its text format); input - The name of the &quot;input&quot; function (the function postgres uses to convert the type from its text format); array_elem - If this is an array, the array elements' oid; base_type - If this is a range type, the base type's oid; comp_elems - If this is a composite type (record), the tuple elements' oid;","ref":"Postgrex.TypeInfo.html","title":"Postgrex.TypeInfo","type":"module"},{"doc":"","ref":"Postgrex.TypeInfo.html#t:t/0","title":"Postgrex.TypeInfo.t/0","type":"type"},{"doc":"Encodes and decodes between PostgreSQL protocol and Elixir values.","ref":"Postgrex.Types.html","title":"Postgrex.Types","type":"module"},{"doc":"Defines a type module with custom extensions and options. Postgrex.Types.define/3 must be called on its own file, outside of any module and function, as it only needs to be defined once during compilation. Type modules are given to Postgrex on start_link via the :types option and are used to control how Postgrex encodes and decodes data coming from Postgrex. For example, to define a new type module with a custom extension called MyExtension while also changing Postgrex 's default behaviour regarding binary decoding, you may create a new file called &quot;lib/my_app/postgrex_types.ex&quot; with the following: Postgrex.Types . define ( MyApp.PostgrexTypes , [ MyExtension ] , [ decode_binary : :reference ] ) The line above will define a new module, called MyApp.PostgrexTypes which can be passed as :types when starting Postgrex. The type module works by rewriting and inlining the extensions' encode and decode expressions in an optimal fashion for postgrex to encode parameters and decode multiple rows at a time. Extensions Extensions is a list of Postgrex.Extension modules or a 2-tuple containing the module and a keyword list. The keyword, defaulting to [] , will be passed to the modules init/1 callback. Extensions at the front of the list will take priority over later extensions when the matching/1 callback returns have conflicting matches. If an extension is not provided for a type then Postgrex will fallback to default encoding/decoding methods where possible. All extensions that ship as part of Postgrex are included out of the box. See Postgrex.Extension for more information on extensions. Options :null - The atom to use as a stand in for postgres' NULL in encoding and decoding. The module attribute @null is registered with the value so that extension can access the value if desired (default: nil ); :decode_binary - Either :copy to copy binary values when decoding with default extensions that return binaries or :reference to use a reference counted binary of the binary received from the socket. Referencing a potentially larger binary can be more efficient if the binary value is going to be garbaged collected soon because a copy is avoided. However the larger binary can not be garbage collected until all references are garbage collected (default: :copy ); :json - The JSON module to encode and decode JSON binaries, calls module.encode_to_iodata!/1 to encode and module.decode!/1 to decode. If nil then no default JSON handling (default: Application.get_env(:postgrex, :json_library, Jason) ); :bin_opt_info - Either true to enable binary optimisation information, or false to disable, for more information see Kernel.SpecialForms.&lt;&lt;&gt;&gt;/1 in Elixir (default: false ); :debug_defaults - Generate debug information when building default extensions so they point to the proper source. Enabling such option will increase the time to compile the type module (default: false ); :moduledoc - The moduledoc to be used for the generated module.","ref":"Postgrex.Types.html#define/3","title":"Postgrex.Types.define/3","type":"function"},{"doc":"PostgreSQL internal identifier that maps to a type. See https://www.postgresql.org/docs/9.4/static/datatype-oid.html .","ref":"Postgrex.Types.html#t:oid/0","title":"Postgrex.Types.oid/0","type":"type"},{"doc":"State used by the encoder/decoder functions","ref":"Postgrex.Types.html#t:state/0","title":"Postgrex.Types.state/0","type":"opaque"},{"doc":"Term used to describe type information","ref":"Postgrex.Types.html#t:type/0","title":"Postgrex.Types.type/0","type":"opaque"},{"doc":"","ref":"Postgrex.Error.html","title":"Postgrex.Error","type":"exception"},{"doc":"","ref":"Postgrex.Error.html#t:t/0","title":"Postgrex.Error.t/0","type":"type"},{"doc":"","ref":"Postgrex.QueryError.html","title":"Postgrex.QueryError","type":"exception"},{"doc":"PostgreSQL driver for Elixir. Documentation: http://hexdocs.pm/postgrex/","ref":"readme.html","title":"Postgrex","type":"extras"},{"doc":"iex&gt; { :ok , pid } = Postgrex . start_link ( hostname : &quot;localhost&quot; , username : &quot;postgres&quot; , password : &quot;postgres&quot; , database : &quot;postgres&quot; ) { :ok , # PID &lt; 0 . 69 . 0 &gt; } iex&gt; Postgrex . query! ( pid , &quot;SELECT user_id, text FROM comments&quot; , [ ] ) % Postgrex.Result { command : :select , empty? : false , columns : [ &quot;user_id&quot; , &quot;text&quot; ] , rows : [ [ 3 , &quot;hey&quot; ] , [ 4 , &quot;there&quot; ] ] , size : 2 } } iex&gt; Postgrex . query! ( pid , &quot;INSERT INTO comments (user_id, text) VALUES (10, &#39;heya&#39;)&quot; , [ ] ) % Postgrex.Result { command : :insert , columns : nil , rows : nil , num_rows : 1 } }","ref":"readme.html#examples","title":"Postgrex - Examples","type":"extras"},{"doc":"Automatic decoding and encoding of Elixir values to and from PostgreSQL's binary format User defined extensions for encoding and decoding any PostgreSQL type Supports transactions, prepared queries and multiple pools via DBConnection Supports PostgreSQL 8.4, 9.0-9.6, and later (hstore is not supported on 8.4)","ref":"readme.html#features","title":"Postgrex - Features","type":"extras"},{"doc":"PostgreSQL Elixir -- -- -- -- -- -- -- -- NULL nil bool true | false char &quot;é&quot; int 42 float 42.0 text &quot;eric&quot; bytea &lt;&lt; 42 &gt;&gt; numeric # Decimal &lt; 42.0 &gt; * date % Date { year : 2013 , month : 10 , day : 12 } time ( tz ) % Time { hour : 0 , minute : 37 , second : 14 } * * timestamp % NaiveDateTime { year : 2013 , month : 10 , day : 12 , hour : 0 , minute : 37 , second : 14 } timestamptz % DateTime { year : 2013 , month : 10 , day : 12 , hour : 0 , minute : 37 , second : 14 , time_zone : &quot;Etc/UTC&quot; } * * interval % Postgrex.Interval { months : 14 , days : 40 , secs : 10920 , microsecs : 315 } array [ 1 , 2 , 3 ] composite type { 42 , &quot;title&quot; , &quot;content&quot; } range % Postgrex.Range { lower : 1 , upper : 5 } multirange % Postgrex.Multirange { ranges : [ % Postgrex.Range { lower : 1 , upper : 5 } , % Postgrex.Range { lower : 20 , upper : 23 } ] } uuid &lt;&lt; 160 , 238 , 188 , 153 , 156 , 11 , 78 , 248 , 187 , 109 , 107 , 185 , 189 , 56 , 10 , 17 &gt;&gt; hstore %{ &quot;foo&quot; =&gt; &quot;bar&quot; } oid types 42 enum &quot;ok&quot; * * * bit &lt;&lt; 1 :: 1 , 0 :: 1 &gt;&gt; varbit &lt;&lt; 1 :: 1 , 0 :: 1 &gt;&gt; tsvector [ % Postgrex.Lexeme { positions : [ { 1 , :A } ] , word : &quot;a&quot; } ] * Decimal ** Timezones will always be normalized to UTC or assumed to be UTC when no information is available, either by PostgreSQL or Postgrex *** Enumerated types (enum) are custom named database types with strings as values. **** Anonymous composite types are decoded (read) as tuples but they cannot be encoded (written) to the database Postgrex does not automatically cast between types. For example, you can't pass a string where a date is expected. To add type casting, support new types, or change how any of the types above are encoded/decoded, you can use extensions.","ref":"readme.html#data-representation","title":"Postgrex - Data representation","type":"extras"},{"doc":"Postgrex comes with JSON support out of the box via the Jason library. To use it, add :jason to your dependencies: { :jason , &quot;~&gt; 1.0&quot; } You can customize it to use another library via the :json_library configuration: config :postgrex , :json_library , SomeOtherLib Once you change the value, you have to recompile Postgrex, which can be done by cleaning its current build: mix deps.clean postgrex --build","ref":"readme.html#json-support","title":"Postgrex - JSON support","type":"extras"},{"doc":"Extensions are used to extend Postgrex' built-in type encoding/decoding. The extensions directory in this project provides implementation for many Postgres' built-in data types. It is also a great example of how to implement your own extensions. For example, you can look at the Date extension as a starting point. Once you defined your extensions, you should build custom type modules, passing all of your extensions as arguments: Postgrex.Types . define ( MyApp.PostgrexTypes , [ MyApp.Postgis.Extensions ] , [ ] ) Postgrex.Types.define/3 must be called on its own file, outside of any module and function, as it only needs to be defined once during compilation. Once a type module is defined, you must specify it on start_link : Postgrex . start_link ( types : MyApp.PostgrexTypes )","ref":"readme.html#extensions","title":"Postgrex - Extensions","type":"extras"},{"doc":"PostgreSQL's wire protocol supports encoding types either as text or as binary. Unlike most client libraries Postgrex uses the binary protocol, not the text protocol. This allows for efficient encoding of types (e.g. 4-byte integers are encoded as 4 bytes, not as a string of digits) and automatic support for arrays and composite types. Unfortunately the PostgreSQL binary protocol transports OID types as integers while the text protocol transports them as string of their name, if one exists, and otherwise as integer. This means you either need to supply oid types as integers or perform an explicit cast (which would be automatic when using the text protocol) in the query. # Fails since $1 is regclass not text. query ( &quot;select nextval($1)&quot; , [ &quot;some_sequence&quot; ] ) # Perform an explicit cast, this would happen automatically when using a # client library that uses the text protocol. query ( &quot;select nextval($1::text::regclass)&quot; , [ &quot;some_sequence&quot; ] ) # Determine the oid once and store it for later usage. This is the most # efficient way, since PostgreSQL only has to perform the lookup once. Client # libraries using the text protocol do not support this. %{ rows : [ { sequence_oid } ] } = query ( &quot;select $1::text::regclass&quot; , [ &quot;some_sequence&quot; ] ) query ( &quot;select nextval($1)&quot; , [ sequence_oid ] )","ref":"readme.html#oid-type-encoding","title":"Postgrex - OID type encoding","type":"extras"},{"doc":"When using PgBouncer with transaction or statement pooling named prepared queries can not be used because the bouncer may route requests from the same postgrex connection to different PostgreSQL backend processes and discards named queries after the transactions closes. To force unnamed prepared queries: Postgrex . start_link ( prepare : :unnamed )","ref":"readme.html#pgbouncer","title":"Postgrex - PgBouncer","type":"extras"},{"doc":"To contribute you need to compile Postgrex from source and test it: $ git clone https://github.com/elixir-ecto/postgrex.git $ cd postgrex $ mix test The tests requires some modifications to your hba file . The path to it can be found by running $ psql -U postgres -c &quot;SHOW hba_file&quot; in your shell. Put the following above all other configurations (so that they override): local all all trust host all postgrex_md5_pw 127.0 . 0.1 / 32 md5 host all postgrex_cleartext_pw 127.0 . 0.1 / 32 password host all postgrex_scram_pw 127.0 . 0.1 / 32 scram - sha - 256 The server needs to be restarted for the changes to take effect. Additionally you need to setup a PostgreSQL user with the same username as the local user and give it trust or ident in your hba file. Or you can export $PGUSER and $PGPASSWORD before running tests. Testing hstore on 9.0 PostgreSQL versions 9.0 does not have the CREATE EXTENSION commands. This means we have to locate the postgres installation and run the hstore.sql in contrib to install hstore . Below is an example command to test 9.0 on OS X with homebrew installed postgres: $ PGVERSION=9.0 PGPATH=/usr/local/share/postgresql9/ mix test","ref":"readme.html#contributing","title":"Postgrex - Contributing","type":"extras"},{"doc":"Copyright 2013 Eric Meadows-Jönsson Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http :// www . apache . org / licenses / LICENSE - 2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html#license","title":"Postgrex - License","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Enhancements Add SCRAM server signature verification Support Multirange extension Support lquery and ltree extensions","ref":"changelog.html#v0-17-0-2023-04-10","title":"Changelog - v0.17.0 (2023-04-10)","type":"extras"},{"doc":"Enhancements Allow the :search_path to be set for new connections","ref":"changelog.html#v0-16-5-2022-09-20","title":"Changelog - v0.16.5 (2022-09-20)","type":"extras"},{"doc":"Enhancements Support Unix sockets in hostname and PGHOST Support infinity value on numerics/decimals (PG14+) Add count to Table.Reader metadata Fix warnings on Elixir v1.15","ref":"changelog.html#v0-16-4-2022-07-29","title":"Changelog - v0.16.4 (2022-07-29)","type":"extras"},{"doc":"Enhancements Implement the Table.Reader protocol for query result","ref":"changelog.html#v0-16-3-2022-04-27","title":"Changelog - v0.16.3 (2022-04-27)","type":"extras"},{"doc":"Enhancements Add :ping_timeout start option Bug fixes Replication streaming can be resumed after reconnect","ref":"changelog.html#v0-16-2-2022-02-21","title":"Changelog - v0.16.2 (2022-02-21)","type":"extras"},{"doc":"Bug fixes Fix inconsistent return type for multiple queries in Postgrex.SimpleConnection and Postgrex.ReplicationConnection . Instead always wrap Postgrex.Result in a list.","ref":"changelog.html#v0-16-1-2022-01-24","title":"Changelog - v0.16.1 (2022-01-24)","type":"extras"},{"doc":"Require Elixir v1.11+. Enhancements Support negative years for date, timestamp and timestampz types Add Postgrex.SimpleConnection and Postgrex.ReplicationConnection Bug fixes Cancel any pending requests before closing the socket Fix possible crash on getting default opts when PGPORT is invalid but a port is given","ref":"changelog.html#v0-16-0-2022-01-23","title":"Changelog - v0.16.0 (2022-01-23)","type":"extras"},{"doc":"Enhancements Support xid8 type introduced in PostgreSQL 13","ref":"changelog.html#v0-15-11-2021-09-26","title":"Changelog - v0.15.11 (2021-09-26)","type":"extras"},{"doc":"Enhancements Define child_spec for Postgrex.Notifications Improve error handling when using multiple endpoints Bug fixes Fix dialyzer warnings Fix invalid type error after failover","ref":"changelog.html#v0-15-10-2021-07-27","title":"Changelog - v0.15.10 (2021-07-27)","type":"extras"},{"doc":"Enhancements Support the new :endpoints and :target_server_type to make it faster to rotate across multiple instances in cases of failovers Bug fixes Do not warn on undefined JSON library Fix bug when a message which is not a row description, data row, command completion or error message occurs and there is buffer remaining to be processed","ref":"changelog.html#v0-15-9-2021-04-24","title":"Changelog - v0.15.9 (2021-04-24)","type":"extras"},{"doc":"Bug fixes Make sure scram authentication method works on Erlang/OTP 24","ref":"changelog.html#v0-15-8-2021-01-19","title":"Changelog - v0.15.8 (2021-01-19)","type":"extras"},{"doc":"Enhancements Add compare and to_string to Postgrex.Interval Bug fixes Allow deallocated queries to be re-prepared","ref":"changelog.html#v0-15-7-2020-10-17","title":"Changelog - v0.15.7 (2020-10-17)","type":"extras"},{"doc":"Enhancements Support Decimal 2.0 Bug fixes Do not keep credentials in state in Postgrex.Notifications","ref":"changelog.html#v0-15-6-2020-09-21","title":"Changelog - v0.15.6 (2020-09-21)","type":"extras"},{"doc":"Enhancements Support optional decoding of infinite timestamps Bug fixes Remove cache statements that cannot be described from cache when on a savepoint transaction (such as when inside the SQL sandbox)","ref":"changelog.html#v0-15-5-2020-06-03","title":"Changelog - v0.15.5 (2020-06-03)","type":"extras"},{"doc":"Enhancements Fix warnings on Elixir v1.11.0-dev","ref":"changelog.html#v0-15-4-2020-05-09","title":"Changelog - v0.15.4 (2020-05-09)","type":"extras"},{"doc":"Enhancements Allow dynamic connection configuration with the :configure for notifications Add :auto_reconnect option for notifications Accept listen commands even if the notifications connection is down or yet to first connect Bug fixes Encode empty arrays in a mechanism compatible with CockroachDB Cleanly terminate connection started with a socket","ref":"changelog.html#v0-15-3-2019-12-11","title":"Changelog - v0.15.3 (2019-12-11)","type":"extras"},{"doc":"Enhancements Improve performance of the bootstrap query","ref":"changelog.html#v0-15-2-2019-10-08","title":"Changelog - v0.15.2 (2019-10-08)","type":"extras"},{"doc":"Enhancements Add support for microseconds in Postgrex.Interval Reduce bootstrap log message to debug and clarify error message","ref":"changelog.html#v0-15-1-2019-09-16","title":"Changelog - v0.15.1 (2019-09-16)","type":"extras"},{"doc":"Postgrex v0.15+ requires Elixir v1.6+. Enhancements Filter bootstrap more efficiently by avoiding loading tables information on startup Only bootstrap new oids during describe: this means reconnects don't run a bootstrap query and describe runs minimal query Parse Postgrex 12beta new version format Raise error when :ssl is required and not started in child_spec/1 Bug fixes Don't encode DateTime, NaiveDateTime or Time unless Calendar.ISO","ref":"changelog.html#v0-15-0-2019-07-18","title":"Changelog - v0.15.0 (2019-07-18)","type":"extras"},{"doc":"Enhancements Make bootstrap query compatible with CockroachDB 19.1 Improve error message when encoding bad tuple","ref":"changelog.html#v0-14-3-2019-05-08","title":"Changelog - v0.14.3 (2019-05-08)","type":"extras"},{"doc":"Bug fixes Fix Elixir deprecation warnings Do not crash when receiving notices during authentication Do not crash when receiving an error (caused by a raise) during query execution","ref":"changelog.html#v0-14-2-2019-04-12","title":"Changelog - v0.14.2 (2019-04-12)","type":"extras"},{"doc":"Bug fixes Bump decimal dependency to avoid runtime warnings","ref":"changelog.html#v0-14-1-2018-11-24","title":"Changelog - v0.14.1 (2018-11-24)","type":"extras"},{"doc":"Enhancements Postgrex.INET will add a /32 netmask to an IPv4 address and a /128 netmask to an IPv6 address during encoding where netmask: nil . When decoding, a /32 netmask (for IPv4) or /128 netmask (for IPv6) will be removed, resulting in netmask: nil for the struct Add :disconnect_on_error_codes which allows Postgrex to automatically disconnect and then reconnect on certain errors. This is useful when using Postgrex against systems that support failover, which would emit certain errors on failover. This change allow those errors to be recovered from transparently Add :cache_statement to Postgrex.query/4 as a built-in statement cache Support scram-sha-256 authentication from PostgreSQL 10 Add Postgrex.prepare_execute/4 Automatically re-prepare queries that failed to encode due to a database type change Backwards incompatible changes Invoke encode_to_iodata! instead of encode! in JSON encoder Remove Postgrex.CIDR and use Postgrex.INET to encode both inet/cidr (as PostgreSQL may perform implicit/explicit casting at any time) Postgrex.Time, Postgrex.Date and Postgrex.Timestamp were deprecated and now have been effectively removed Postgrex.execute/4 now always returns the prepared query :pool_timeout is removed in favor of :queue_target and :queue_interval . See DBConnection.start_link/2 for more information","ref":"changelog.html#v0-14-0-2018-10-29","title":"Changelog - v0.14.0 (2018-10-29)","type":"extras"},{"doc":"Enhancements Support custom range domains Support custom array domains Add support for UNIX domain sockets via the :socket_dir option Remove warnings on Elixir v1.6 Bug fixes Fix encoding of empty ranges Fix Postgrex.Path open/closed byte parity","ref":"changelog.html#v0-13-4-2018-01-25","title":"Changelog - v0.13.4 (2018-01-25)","type":"extras"},{"doc":"Enhancements Reload types on unknown oid during prepare Bug fixes Fix default timeout for connection process from 5000s to 15000s","ref":"changelog.html#v0-13-3-2017-05-31","title":"Changelog - v0.13.3 (2017-05-31)","type":"extras"},{"doc":"Bug fixes Do not build invalid dates at compilation time on Elixir master","ref":"changelog.html#v0-13-2-2017-03-05","title":"Changelog - v0.13.2 (2017-03-05)","type":"extras"},{"doc":"Enhancements Allow naming Postgrex.Notifications server Provide tsvector and lexeme support using binary formats Bug fixes Fix encoding of Decimal values that would be wrong in certain circumstances Add :crypto to applications Specify proper Elixir dependency Restore compatibility with postgres versions prior to 8.4 and with redshift","ref":"changelog.html#v0-13-1-2017-02-20","title":"Changelog - v0.13.1 (2017-02-20)","type":"extras"},{"doc":"Enhancements Support built-in geometry types Fallback to PGDATABASE system env for the database Support bit and varbit types Add postgres error code to error messages Support unprepared when using a stream :connect_timeout and :handshake_timeout to configure TCP connect and handshake timeouts Improve numeric encode/decode Bug fixes Quote channel on listen/unlisten Check datetime structs available before defining calendar extension Backoff all awaiting connections if a bootstrap fails to prevent timeout loop Handle idle admin shutdown of postgres backend Fix rebootstrap query to be O(Nlog(N)) instead of O(N^2) Fix encoding of numerical values Backwards incompatible changes :copy_data query option is no longer supported and data can only be copied to the database using a collectable Query struct has removed encoders/decoders and changed param_info/result_info values Extensions now use a new encoder/decoder API based on quoted expressions The :extensions , :decode_binary and :null options in start_link are no longer supported in favor of defining custom types with Postgrex.Types.define(module, extra_extensions, options) . Postgrex.Types.define/3 must be called on its own file, outside of any module and function, as it only needs to be defined once during compilation.","ref":"changelog.html#v0-13-0-2016-12-17","title":"Changelog - v0.13.0 (2016-12-17)","type":"extras"},{"doc":"Enhancements Support special &quot;char&quot; type Bug fixes Limit re-bootstrap to one connection at a time Fix re-bootstrap of new composite types that use old types","ref":"changelog.html#v0-12-1-2016-09-29","title":"Changelog - v0.12.1 (2016-09-29)","type":"extras"},{"doc":"Enhancements Raise DBConnection.ConnectionError on connection error Use send encoding to determine citext encoding Use Map in favor of deprecated modules (to avoid warnings on v1.4) Run rebootstrap test synchronously on every connect Add support for Elixir 1.3+ Calendar types","ref":"changelog.html#v0-12-0-2016-09-06","title":"Changelog - v0.12.0 (2016-09-06)","type":"extras"},{"doc":"Enhancements Add support for COPY TO STDOUT and COPY FROM STDIN Support packets bigger than 64MB Introduce mode: :savepoint for prepare/execute/close that allows wrapping a request in a savepoint so that an error does not fail the transaction Introduce streaming queries Add :decode_binary option which is either :copy (default) or :reference . Bug fixes Consistently convert the port number to integer Remove type server entry on disconnect","ref":"changelog.html#v0-11-2-2016-06-16","title":"Changelog - v0.11.2 (2016-06-16)","type":"extras"},{"doc":"Enhancements Support PgBouncer transaction/statement pooling Include more information in error messages Add support for built-in postgres point type Add Postgrex.child_spec/1 Allow custom encoding/decoding of postgres' NULL on a per query basis Bug fixes Correctly pad decimal digits during encoding","ref":"changelog.html#v0-11-1-2016-02-15","title":"Changelog - v0.11.1 (2016-02-15)","type":"extras"},{"doc":"Enhancements Rely on DBConnection. This means better performance by copying less data between processes, faster encoding/decoding, support for transactions, after_connect hooks, connection backoff, logging, prepared queries, the ability to use both Poolboy and Sojourn as pools out of the box, and more Backwards incompatible change Connection API from Postgrex.Connection has been moved to Postgrex Notifications API from Postgrex.Connection has been moved to Postgrex.Notifications","ref":"changelog.html#v0-11-0-2016-01-21","title":"Changelog - v0.11.0 (2016-01-21)","type":"extras"},{"doc":"Enhancements Improve error message on encoding/decoding failures Add network types such as: inet , cidr and macaddr Improve TCP error messages Support PGPORT environment variable Improve decoding performance by caching extension information Improve query performance by decoding in the client process hence not blocking the connection Raise if number of parameters to query is wrong Bug fixes Correctly handle errors in connection initialization with sync_connect: true Do not fail on custom error codes Correctly handle large number of parameters, also fixes some protocol issues where unsigned integers were treated as signed","ref":"changelog.html#v0-10-0-2015-11-17","title":"Changelog - v0.10.0 (2015-11-17)","type":"extras"},{"doc":"Enhancements Revert client side decoding as affects performance negatively (around 15% slower) Cast floats and integers to decimal if a decimal is requested","ref":"changelog.html#v0-9-1-2015-07-14","title":"Changelog - v0.9.1 (2015-07-14)","type":"extras"},{"doc":"Enhancements Cached type bootstrapping for less memory usage and faster connection set up The result set is now decoded in the calling process to reduce time spent in the connection process Add a decode: :manual option to Postgrex.query/4 and the function Postgrex.decode/2 for manually decoding the result Add :sync_connect option to Postgrex.start_link/1 Bug fixes Correctly handle extension types created inside schemas Backwards incompatible changes Each row in Postgrex.Result.rows is now a list of columns instead of a tuple","ref":"changelog.html#v0-9-0-2015-07-12","title":"Changelog - v0.9.0 (2015-07-12)","type":"extras"},{"doc":"Bug fixes Fix version detection","ref":"changelog.html#v0-8-4-2015-06-24","title":"Changelog - v0.8.4 (2015-06-24)","type":"extras"},{"doc":"Enhancements Add Postgrex.Extensions.JSON extension for json and jsonb types Set suitable TCP buffer size automatically","ref":"changelog.html#v0-8-3-2015-06-22","title":"Changelog - v0.8.3 (2015-06-22)","type":"extras"},{"doc":"Enhancements Add :socket_options option to Postgrex.start_link/1 Improved performance regarding binary handling Add hstore support Backwards incompatible changes Remove :async_connect option and make it the default","ref":"changelog.html#v0-8-2-2015-06-01","title":"Changelog - v0.8.2 (2015-06-01)","type":"extras"},{"doc":"Enhancements Keep the postgres error code in :pg_code Support oid types and all its aliases (regclass etc) Backwards incompatible changes Rename :msec field to :usec on Postgrex.Time and Postgrex.Timestamp Bug fixes Fix numeric encoding for fractional numbers with less digits than the numeric base Support encoding timetz type Fix time and timestamp off-by-one bounds","ref":"changelog.html#v0-8-1-2015-04-09","title":"Changelog - v0.8.1 (2015-04-09)","type":"extras"},{"doc":"Enhancements Add extensions Encode/decode ranges generically Add bounds when encoding integer types to error instead of overflowing the integer Log unhandled PostgreSQL errors (when it cant be replied to anyone) Add support for enum types Add support for citext type Add microseconds to times and timestamps Add the ability to rebootstrap types for an open connection Backwards incompatible changes Remove the support for type-hinted queries Remove encoder, decoder and formatter functions, use extensions instead Use structs for dates, times, timestamps, interval and ranges Change the default timeout for all operations to 5000ms Show PostgreSQL error codes as their names instead","ref":"changelog.html#v0-8-0-2015-02-26","title":"Changelog - v0.8.0 (2015-02-26)","type":"extras"},{"doc":"Enhancements Add asynchronous notifications through listen and unlisten Add support for range types Add support for uuid type Add :async_connect option to start_link/1 Bug fixes Fix encoding nil values in arrays and composite types","ref":"changelog.html#v0-7-0-2015-01-20","title":"Changelog - v0.7.0 (2015-01-20)","type":"extras"},{"doc":"Enhancements Queries can be constructed of iodata Support &quot;type hinted&quot; queries to save one client-server round trip which will reduce query latency Backwards incompatible changes Postgrex.Error postgres field is converted from keyword list to map Postgrex.Connect.query params parameter is no longer optional (pass an empty list if query has no parameters) The timeout parameter for all functions have been moved to a keyword list with the key :timeout","ref":"changelog.html#v0-6-0-2014-09-07","title":"Changelog - v0.6.0 (2014-09-07)","type":"extras"},{"doc":"Enhancements Reduce the amount of intermediary binaries constructed with the help of iodata","ref":"changelog.html#v0-5-5-2014-08-20","title":"Changelog - v0.5.5 (2014-08-20)","type":"extras"},{"doc":"","ref":"changelog.html#v0-5-4-2014-08-04","title":"Changelog - v0.5.4 (2014-08-04)","type":"extras"},{"doc":"","ref":"changelog.html#v0-5-3-2014-07-13","title":"Changelog - v0.5.3 (2014-07-13)","type":"extras"},{"doc":"","ref":"changelog.html#v0-5-2-2014-06-18","title":"Changelog - v0.5.2 (2014-06-18)","type":"extras"},{"doc":"Backwards incompatible changes Postgrex.Error exception converted to struct","ref":"changelog.html#v0-5-1-2014-05-24","title":"Changelog - v0.5.1 (2014-05-24)","type":"extras"},{"doc":"Backwards incompatible changes Postgrex.Result and Postgrex.TypeInfo converted to structs","ref":"changelog.html#v0-5-0-2014-05-01","title":"Changelog - v0.5.0 (2014-05-01)","type":"extras"},{"doc":"Enhancements Add timeouts to all synchronous calls. When a timeout is hit an exit error will be raised in the caller process and the connection process will exit Add automatic fallback to environment variables PGUSER , PGHOST and PGPASSWORD","ref":"changelog.html#v0-4-2-2014-04-21","title":"Changelog - v0.4.2 (2014-04-21)","type":"extras"},{"doc":"Enhancements Numerics decode and encode to Decimal","ref":"changelog.html#v0-4-0-2014-01-16","title":"Changelog - v0.4.0 (2014-01-16)","type":"extras"},{"doc":"Enhancements Compact state before printing to logs and hide password Concurrency support, safe to use connection from multiple processes concurrently","ref":"changelog.html#v0-3-1-2014-01-15","title":"Changelog - v0.3.1 (2014-01-15)","type":"extras"},{"doc":"Bug fixes Don't try to decode values of text format Backwards incompatible changes Types are stored as binaries instead of atoms, update your custom encoders and decoders","ref":"changelog.html#v0-3-0-2013-12-16","title":"Changelog - v0.3.0 (2013-12-16)","type":"extras"},{"doc":"Enhancements Add support for SSL Bug fixes Fix decoding of unknown type when using custom decoder","ref":"changelog.html#v0-2-1-2013-12-10","title":"Changelog - v0.2.1 (2013-12-10)","type":"extras"},{"doc":"Enhancements Floats handles NaN, inf and -inf Add support for numerics Custom encoders and decoders works on elements in arrays Add support for composite types Add functions that raise on error Bug fixes INSERT query works with extended query parameters Return proper num_rows on PostgreSQL 8.4 Fix race condition Backwards incompatible changes Simplify custom decoding and encoding with default function","ref":"changelog.html#v0-2-0-2013-11-14","title":"Changelog - v0.2.0 (2013-11-14)","type":"extras"},{"doc":"First release!","ref":"changelog.html#v0-1-0-2013-10-14","title":"Changelog - v0.1.0 (2013-10-14)","type":"extras"}]